# Before `make install' is performed this script should be runnable with
# `make test'. After `make install' it should work as `perl test.pl'

#########################

# change 'tests => 1' to 'tests => last_test_to_print';

use Test;
BEGIN { plan tests => 1 };
use Debug;
ok(1); # If we made it this far, we're ok.

#########################

# Insert your test code below, the Test module is use()ed here so read
# its man page ( perldoc Test ) for help writing this test script.

my $debug = new Debug();
$debug->initialize();
$debug->message("Testing 123");
$debug->timeStamp();
$debug->message("Testing 123");
$debug->detail();
$debug->message("Testing 123");
